﻿/*
 * Created by SharpDevelop.
 * User: pshai
 * Date: 6.7.2016
 * Time: 10.24
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TradeServer.ScriptingDriver.DataObjects;
using TradeServer.ScriptingDriver.Interfaces;
using TicTacTec.TA.Library;

namespace pshaisMAv3
{
    public class pshaisMAv3 : IIndicatorScript
    {
        int custom_interval = 1;

        int ma_length1 = 10, ma_length2 = 50;
        Core.MAType ma_type1 = Core.MAType.Sma, ma_type2 = Core.MAType.Ema;

        double swing_prc;
        int deviation = 0;

        IndicatorResult last_result;

        decimal buy_position, sell_position;
        //decimal buy_threshold = 1, sell_threshold= 1;

        string price_source1 = Utils.SOURCE_CLOSE;
        string price_source2 = Utils.SOURCE_CLOSE;

        const string SIGNAL_TYPE1 = "cross";
        const string SIGNAL_TYPE2 = "dir-fast";
        const string SIGNAL_TYPE3 = "dir-slow";
        const string SIGNAL_TYPE4 = "dir-both";

        string signal_type = SIGNAL_TYPE1;

        bool zero_lag1 = false, zero_lag2 = false;
        int lag1 { get { return (ma_length1 - 1) / 2; } }
        int lag2 { get { return (ma_length2 - 1) / 2; } }

        double ma_value1, ma_value2;

        #region IIndicatorScript implementation
        public void Init()
        {
            
        }

        public IndicatorResult GetResult(IndicatorContext context)
        {
            try
            {
                IPriceInstrument instrument = context.PriceInstrument;
                
                int position = instrument.Close.Count() - (1 + context.Offset + deviation);
                
                var high = instrument.High;
                var low = instrument.Low;
                var close = instrument.Close;
                var open = instrument.Open;

                double[] source1 = Utils.GetSource(instrument, price_source1);
                double[] source2 = Utils.GetSource(instrument, price_source2);
                
                var position1 = position;
                var position2 = position;

                if (zero_lag1)
                {
                    source1 = prepZlema(source1, lag1);
                    position1 -= lag1;
                }

                if (zero_lag2)
                {
                    source2 = prepZlema(source2, lag2);
                    position2 -= lag2;
                }

                // ma1
                var lb1 = Core.MovingAverageLookback(ma_length1, ma_type1);
                int ma_begin1, ma_len1;
                var ma_out1 = new double[lb1 + 151];
                var ret1 = Core.MovingAverage(position1 - (lb1 + 150), position1, source1, ma_length1, ma_type1, out ma_begin1, out ma_len1, ma_out1);

                // ma2
                var lb2 = Core.MovingAverageLookback(ma_length2, ma_type2);
                int ma_begin2, ma_len2;
                var ma_out2 = new double[lb2 + 151];
                var ret2 = Core.MovingAverage(position2 - (lb2 + 150), position2, source2, ma_length2, ma_type2, out ma_begin2, out ma_len2, ma_out2);

                if (ret1 == Core.RetCode.Success && ret2 == Core.RetCode.Success
                  && ma_out1.Length > 0 && ma_len1 > 0
                  && ma_out2.Length > 0 && ma_len2 > 0)
                {
                    ma_value1 = ma_out1[ma_len1 - 1];
                    ma_value2 = ma_out2[ma_len2 - 1];
                    var ma_prev1 = ma_out1[ma_len1 - 2];
                    var ma_prev2 = ma_out2[ma_len2 - 2];

                    
                    var result = IndicatorResult.StayNoPosition;

                    switch (signal_type)
                    {
                        default:
                        case SIGNAL_TYPE1:
                            if (ma_value1 * (1 - swing_prc) > ma_value2)
                                result = IndicatorResult.BuyLong;
                            if (ma_value1 * (1 + swing_prc) < ma_value2)
                                result = IndicatorResult.SellShort;
                            break;

                        // MA Direction signal of MA1
                        case SIGNAL_TYPE2:
                            if (ma_value1 * (1 - swing_prc) > ma_prev1) // going up, swing prc does affect
                                result = IndicatorResult.BuyLong;
                            if (ma_value1 * (1 + swing_prc) < ma_prev1) // going down, swing prc does affect
                                result = IndicatorResult.SellShort;
                            break;

                        // MA Direction signal of MA2
                        case SIGNAL_TYPE3:
                            if (ma_value2 * (1 - swing_prc) > ma_prev2) // going up, swing prc does affect
                                result = IndicatorResult.BuyLong;
                            if (ma_value2 * (1 + swing_prc) < ma_prev2) // going down, swing prc does affect
                                result = IndicatorResult.SellShort;
                            break;

                        // MA Direction signal of both
                        case SIGNAL_TYPE4:
                            if (ma_value1 * (1 - swing_prc) > ma_prev1 && ma_value2 * (1 - swing_prc) > ma_prev2) // going up, swing prc does affect
                                result = IndicatorResult.BuyLong;
                            if (ma_value1 * (1 + swing_prc) < ma_prev1 && ma_value2 * (1 + swing_prc) < ma_prev2) // going down, swing prc does affect
                                result = IndicatorResult.SellShort;
                            break;
                    }
                    


                    last_result = result;
                    return result;
                }


            }
            catch (Exception ex) // This try catch block is an highly recommend practice
            {
                context.Logger.LogToFile(string.Format("Exception detected: {0}", ex));
            }

            return IndicatorResult.StayNoPosition; // Default output
        }


        public List<ScriptParameter> GetParameters()
        {

            var rtn = new List<ScriptParameter>
            {
                new ScriptParameter("MA1 Length", ScriptParameterType.Integer, ma_length1.ToString()),
                new ScriptParameter("MA1 Type", ScriptParameterType.String, ma_type1.ToString()),
                new ScriptParameter("MA1 Source", ScriptParameterType.String, price_source1),
                new ScriptParameter("MA1 Zero-Lag? (y/n)", ScriptParameterType.String, zero_lag1 ? "y" : "n"),
                new ScriptParameter("MA2 Length", ScriptParameterType.Integer, ma_length2.ToString()),
                new ScriptParameter("MA2 Type", ScriptParameterType.String, ma_type2.ToString()),
                new ScriptParameter("MA2 Source", ScriptParameterType.String, price_source2),
                new ScriptParameter("MA2 Zero-Lag? (y/n)", ScriptParameterType.String, zero_lag2 ? "y" : "n"),
                new ScriptParameter("Signal Type", ScriptParameterType.String, signal_type),
                new ScriptParameter("Swing %", ScriptParameterType.Decimal, swing_prc.ToString()),
                new ScriptParameter("Deviation", ScriptParameterType.Integer, deviation.ToString()),
            };

            return rtn;
        }
        public void SetParameters(Dictionary<string, object> parameters)
        {
            ma_length1 = ParseInt(parameters["MA1 Length"], ma_length1);
            ma_type1 = ParseMAType(parameters["MA1 Type"]);
            price_source1 = ParseString(parameters["MA1 Source"], price_source1);
            zero_lag1 = ParseString(parameters["MA1 Zero-Lag? (y/n)"], zero_lag1 ? "y" : "n") == "y";

            ma_length2 = ParseInt(parameters["MA2 Length"], ma_length2);
            ma_type2 = ParseMAType(parameters["MA2 Type"]);
            price_source2 = ParseString(parameters["MA2 Source"], price_source2);
            zero_lag2 = ParseString(parameters["MA2 Zero-Lag? (y/n)"], zero_lag2 ? "y" : "n") == "y";

            signal_type = ParseString(parameters["Signal Type"], signal_type);

            swing_prc = ParseDouble(parameters["Swing %"], swing_prc);

            deviation = ParseInt(parameters["Deviation"], deviation);
        }
        public List<ChartRecord> GetChartLines()
        {
            List<ChartRecord> chartLines = new List<ChartRecord>
            {
                new ChartRecord() { ChartIndex = 0, Name = string.Format("{0}({1})", ma_type1, ma_length1), HexLineColor = "#FF4444" },
                new ChartRecord() { ChartIndex = 0, Name = string.Format("{0}({1})", ma_type2, ma_length2), HexLineColor = "#FF8822" }
            };

            return chartLines;
        }
        public List<decimal> GetChartData()
        {
            var chartData = new List<decimal>
            {
                Convert.ToDecimal(ma_value1),
                Convert.ToDecimal(ma_value2)
            };

            return chartData;
        }
        public string Name
        {
            get
            {
                return "pshai's Moving Averages v3";
            }
        }
        public int TicksBack
        {
            get
            {
                // Return the longest lookback
				var lb1 = Core.MovingAverageLookback(ma_length1, ma_type1);
				var lb2 = Core.MovingAverageLookback(ma_length2, ma_type2);
				return Math.Max(lb1, lb2) + 200 + deviation + Math.Max(lag1, lag2);
            }
        }
        #endregion


        double[] prepZlema(double[] price, int lag)
        {
            var price2 = new double[price.Length - lag];

            for (int i = lag; i < price.Length; i++)
            {
                var i2 = i - lag;

                price2[i2] = price[i] + (price[i] - price[i - lag]);
            }

            return price2;
        }

        public static Core.MAType ParseMAType(object value)
        {
            Core.MAType val;
            switch (value.ToString().ToLower())
            {
                case "sma":
                    val = Core.MAType.Sma;
                    break;
                case "ema":
                    val = Core.MAType.Ema;
                    break;
                case "dema":
                    val = Core.MAType.Dema;
                    break;
                case "tema":
                    val = Core.MAType.Tema;
                    break;
                case "trima":
                    val = Core.MAType.Trima;
                    break;
                case "t3":
                    val = Core.MAType.T3;
                    break;
                case "mama":
                    val = Core.MAType.Mama;
                    break;
                case "kama":
                    val = Core.MAType.Kama;
                    break;
                case "wma":
                    val = Core.MAType.Wma;
                    break;
                default:
                    val = Core.MAType.Sma;
                    break;
            }

            return val;
        }

        int ParseInt(object value, int lastValue)
        {
            try
            {
                return Convert.ToInt32(value.ToString());
            }
            catch
            {
                return lastValue;
            }
        }

        double ParseDouble(object value, double lastValue)
        {
            var numStyle = System.Globalization.NumberStyles.Any;
            var cult = System.Globalization.CultureInfo.InvariantCulture;

            try
            {
                double output;
                double.TryParse(
                    value.ToString().Replace(',', '.'),
                    numStyle,
                    cult,
                    out output);
                return output;
            }
            catch
            {
                return lastValue;
            }
        }

        string ParseString(object value, string lastValue)
        {
            try
            {
                return Convert.ToString(value);
            }
            catch
            {
                return lastValue;
            }
        }

        #region Candlestick converter
        //
        // Converts candlestick data to another interval
        List<double[]> instr(IndicatorContext context, int mult, out int countOut, bool fullCandlesOnly = true)
        {
            // Get raw input data
            DateTime[] timestamp = context.PriceInstrument.Timestamps;
            double[] high = context.PriceInstrument.High;
            double[] low = context.PriceInstrument.Low;
            double[] open = context.PriceInstrument.Open;
            double[] close = context.PriceInstrument.Close;
            double[] volume = context.PriceInstrument.Volume;
            int interval = context.Interval;
            int compInterval = interval * mult;

            // Output data
            var len = timestamp.Length;
            var ret = new List<double[]>();
            var timestampOut = new List<DateTime>();
            var highOut = new List<double>();
            var lowOut = new List<double>();
            var openOut = new List<double>();
            var closeOut = new List<double>();
            var volumeOut = new List<double>();
            countOut = 0;

            // Datetime stuff
            var curTime = DateTime.MinValue;
            var ts = TimeSpan.FromMinutes(compInterval);
            DateTime lastTimestamp = timestamp.Last();
            DateTime lastRoundedTimestamp = RoundDown(lastTimestamp, ts);
            DateTime time;

            for (int i = 0; i < len; i++)
            {
                // Round down the time to match new interval
                try
                {
                    var tempStamp = timestamp[i];

                    // Skip to avoid incomplete candles
                    if (fullCandlesOnly && tempStamp > lastRoundedTimestamp)
                        continue;

                    time = RoundDown(tempStamp, ts);
                }
                catch
                {
                    throw new Exception("DateTime RoundDown failed!");
                }


                if (time > curTime) // Create new "candle"
                {
                    try
                    {
                        timestampOut.Add(time);
                        highOut.Add(high[i]);
                        lowOut.Add(low[i]);
                        openOut.Add(open[i]);
                        closeOut.Add(close[i]);
                        volumeOut.Add(volume[i]);
                        countOut = 1;

                        curTime = time;
                    }
                    catch
                    {
                        throw new Exception("Creating candle failed!");
                    }
                }
                else // ...or edit data for current candle
                {
                    try
                    {
                        var pos = timestampOut.Count - 1;

                        highOut[pos] = Math.Max(highOut[pos], high[i]);
                        lowOut[pos] = Math.Min(lowOut[pos], low[i]);
                        closeOut[pos] = close[i];
                        volumeOut[pos] += volume[i];
                        countOut++;
                    }
                    catch
                    {
                        throw new Exception("Editing candle failed!");
                    }
                }
            }

            // Save data and gtfo
            ret.Add(highOut.ToArray());
            ret.Add(lowOut.ToArray());
            ret.Add(openOut.ToArray());
            ret.Add(closeOut.ToArray());
            ret.Add(volumeOut.ToArray());

            return ret;
        }

        //
        // Converts longer interval data to match shorter interval
        double[] reso(double[] data, int intRes, int lastTickSize)
        {
            // Convert only if needed
            if (intRes > 1)
            {
                var tmp = new List<double>();
                foreach (var d in data)
                    if (d > 0)
                        tmp.Add(d);

                data = tmp.ToArray();
                var nLen = data.Length * intRes;
                var ret = new double[nLen];
                for (int i = 0; i < data.Length; i++)
                {
                    if (i < data.Length - 1)
                    {
                        for (int j = 0; j < intRes; j++)
                            ret[i * intRes + j] = data[i];
                    }
                    else
                    {
                        // if we dont have full candle in the multiplied data, then we cant stretch the last tick to full length
                        for (int j = 0; j < lastTickSize; j++)
                            ret[i * intRes + j] = data[i];
                    }

                }
                return ret;
            }
            return data;
        }

        // Rounding functions for DateTime
        public static DateTime RoundUp(DateTime dt, TimeSpan ts)
        {
            return Round(dt, ts, true);
        }

        public static DateTime RoundDown(DateTime dt, TimeSpan ts)
        {
            return Round(dt, ts, false);
        }

        private static DateTime Round(DateTime dt, TimeSpan ts, bool up)
        {
            var remainder = dt.Ticks % ts.Ticks;
            if (remainder == 0)
            {
                return dt;
            }

            long delta;
            if (up)
            {
                delta = ts.Ticks - remainder;
            }
            else
            {
                delta = -remainder;
            }

            return dt.AddTicks(delta);
        }
        #endregion
    }

    static class Utils
    {
        public const string SOURCE_CLOSE = "close";
        public const string SOURCE_OPEN = "open";
        public const string SOURCE_HIGH = "high";
        public const string SOURCE_LOW = "low";
        public const string SOURCE_HL2 = "hl2";
        public const string SOURCE_HLC3 = "hlc3";
        public const string SOURCE_OHLC4 = "ohlc4";

        public static double[] GetSource(IPriceInstrument instrument, string source_id)
        {
            switch (source_id)
            {
                default:
                    return instrument.Close;

                case SOURCE_OPEN:
                    return instrument.Open;

                case SOURCE_HIGH:
                    return instrument.High;

                case SOURCE_LOW:
                    return instrument.Low;

                case SOURCE_HL2:
                    return GetHL2(instrument);

                case SOURCE_HLC3:
                    return GetHLC3(instrument);

                case SOURCE_OHLC4:
                    return GetOHLC4(instrument);
            }
        }

        public static double[] GetHL2(IPriceInstrument instrument)
        {
            var len = instrument.Close.Length;
            var rtn = new double[len];
            for (int i = 0; i < len; i++)
            {
                rtn[i] = (instrument.High[i] + instrument.Low[i]) / 2;
            }
            return rtn;
        }
        public static double[] GetHLC3(IPriceInstrument instrument)
        {
            var len = instrument.Close.Length;
            var rtn = new double[len];
            for (int i = 0; i < len; i++)
            {
                rtn[i] = (instrument.High[i] + instrument.Low[i] + instrument.Close[i]) / 3;
            }
            return rtn;
        }
        public static double[] GetOHLC4(IPriceInstrument instrument)
        {
            var len = instrument.Close.Length;
            var rtn = new double[len];
            for (int i = 0; i < len; i++)
            {
                rtn[i] = (instrument.Open[i] + instrument.High[i] + instrument.Low[i] + instrument.Close[i]) / 4;
            }
            return rtn;
        }
    }
}